{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: frmMain.pas, released 2000-11-11.

The Original Code is part of the SimpleIDEDemo project, written by
Michael Hieke for the SynEdit component suite.
All Rights Reserved.

Contributors to the SynEdit project are listed in the Contributors.txt file.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

$Id: frmMain.pas,v 1.7 2004/04/24 17:04:54 markonjezic Exp $

You may retrieve the latest version of this file at the SynEdit home page,
located at http://SynEdit.SourceForge.net

Known Issues:
-------------------------------------------------------------------------------}

unit IDE;

{$I SynEdit.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ActnList, ImgList, ComCtrls, ToolWin, SynEdit, SynEditHighlighter,
  SynHighlighterPas, uSimpleIDEDebugger, Menus, SynEditTypes, StdCtrls,
  sSkinProvider, SynCompletionProposal;

type
  TSimpleIDEMainForm = class(TForm)
    tbDebug: TToolBar;
    tbtnRun: TToolButton;
    tbtnStep: TToolButton;
    tbtnGotoCursor: TToolButton;
    tbtnPause: TToolButton;
    tbtnStop: TToolButton;
    imglActions: TImageList;
    actlMain: TActionList;
    actDebugRun: TAction;
    actDebugStep: TAction;
    actDebugGotoCursor: TAction;
    actDebugPause: TAction;
    actDebugStop: TAction;
    ToolButton1: TToolButton;
    tbtnToggleBreakpoint: TToolButton;
    tbtnClearAllBreakpoints: TToolButton;
    actToggleBreakpoint: TAction;
    actClearAllBreakpoints: TAction;
    SynEditor: TSynEdit;
    SynPasSyn1: TSynPasSyn;
    imglGutterGlyphs: TImageList;
    MainMenu1: TMainMenu;
    mDebug: TMenuItem;
    miDebugRun: TMenuItem;
    miDebugStep: TMenuItem;
    miDebugGotoCursor: TMenuItem;
    miDebugPause: TMenuItem;
    miDebugStop: TMenuItem;
    N1: TMenuItem;
    miToggleBreakpoint: TMenuItem;
    miClearBreakpoints: TMenuItem;
    Statusbar: TStatusBar;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    Edit1: TEdit;
    sSkinProvider1: TsSkinProvider;
    Edit2: TEdit;
    scpDemo: TSynCompletionProposal;
    mmoInsert: TMemo;
    mmoItem: TMemo;
    scpParams: TSynCompletionProposal;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure SynEditorSpecialLineColors(Sender: TObject; Line: Integer;
      var Special: Boolean; var FG, BG: TColor);
    procedure actDebugRunExecute(Sender: TObject);
    procedure actDebugRunUpdate(Sender: TObject);
    procedure actDebugStepExecute(Sender: TObject);
    procedure actDebugStepUpdate(Sender: TObject);
    procedure actDebugGotoCursorExecute(Sender: TObject);
    procedure actDebugGotoCursorUpdate(Sender: TObject);
    procedure actDebugPauseExecute(Sender: TObject);
    procedure actDebugPauseUpdate(Sender: TObject);
    procedure actDebugStopExecute(Sender: TObject);
    procedure actDebugStopUpdate(Sender: TObject);
    procedure actToggleBreakpointExecute(Sender: TObject);
    procedure actToggleBreakpointUpdate(Sender: TObject);
    procedure actClearAllBreakpointsExecute(Sender: TObject);
    procedure actClearAllBreakpointsUpdate(Sender: TObject);
    procedure SynEditorGutterClick(Sender: TObject; Button: TMouseButton;
      X, Y, Line: Integer; Mark: TSynEditMark);
    procedure ToolButton2Click(Sender: TObject);
    procedure ToolButton3Click(Sender: TObject);
    procedure ToolButton4Click(Sender: TObject);
    procedure scpParamsExecute(Kind: TSynCompletionType; Sender: TObject;
      var CurrentInput: string; var x, y: Integer; var CanExecute: Boolean);
  private
    LookupList : TStringList;
    fCurrentLine: integer;
    fDebugger: TSampleDebugger;
    procedure DebuggerBreakpointChange(Sender: TObject; ALine: integer);
    procedure DebuggerCurrentLineChange(Sender: TObject);
    procedure DebuggerStateChange(Sender: TObject; OldState,
      NewState: TDebuggerState);
    procedure DebuggerYield(Sender: TObject);
    procedure PaintGutterGlyphs(ACanvas: TCanvas; AClip: TRect;
      FirstLine, LastLine: integer);
    procedure SetCurrentLine(ALine: integer);
      public
    { Public declarations }
  end;
var
  SimpleIDEMainForm: TSimpleIDEMainForm;
  locline, lookup: String;
    TmpX, savepos, StartX,
    ParenCounter,
    TmpLocation    : Integer;
    FoundMatch     : Boolean;
     implementation

{$R *.DFM}

{ TGutterMarkDrawPlugin }

type
  TDebugSupportPlugin = class(TSynEditPlugin)
  protected
    fForm: TSimpleIDEMainForm;
    procedure AfterPaint(ACanvas: TCanvas; const AClip: TRect;
      FirstLine, LastLine: integer); override;
    procedure LinesInserted(FirstLine, Count: integer); override;
    procedure LinesDeleted(FirstLine, Count: integer); override;
  public
    constructor Create(AForm: TSimpleIDEMainForm);
  end;

constructor TDebugSupportPlugin.Create(AForm: TSimpleIDEMainForm);
begin
  inherited Create(AForm.SynEditor);
  fForm := AForm;
end;

procedure TDebugSupportPlugin.AfterPaint(ACanvas: TCanvas; const AClip: TRect;
  FirstLine, LastLine: integer);
begin
  fForm.PaintGutterGlyphs(ACanvas, AClip, FirstLine, LastLine);
end;

procedure TDebugSupportPlugin.LinesInserted(FirstLine, Count: integer);
begin
// Note: You will need this event if you want to track the changes to
//       breakpoints in "Real World" apps, where the editor is not read-only
end;

procedure TDebugSupportPlugin.LinesDeleted(FirstLine, Count: integer);
begin
// Note: You will need this event if you want to track the changes to
//       breakpoints in "Real World" apps, where the editor is not read-only
end;

{ TSimpleIDEMainForm }

procedure TSimpleIDEMainForm.FormCreate(Sender: TObject);
var
  Settings: TStringList;
begin
  fCurrentLine := -1;
  fDebugger := TSampleDebugger.Create;
  with fDebugger do begin
    OnBreakpointChange := DebuggerBreakpointChange;
    OnCurrentLineChange := DebuggerCurrentLineChange;
    OnStateChange := DebuggerStateChange;
    OnYield := DebuggerYield;
          end;

  TDebugSupportPlugin.Create(Self);

  Settings := TStringList.Create;
  try
    SynPasSyn1.EnumUserSettings(Settings);
    if Settings.Count > 0 then
      SynPasSyn1.UseUserSettings(Settings.Count - 1);
  finally
    Settings.Free;
  end;
  SynEditor.Text := SampleSource;
// Carrega Insert.txt
 mmoInsert.Clear;
 mmoInsert.Lines.LoadFromFile('C:\HMI G3\Inset.txt'); //
 // Carrega Insert.txt
mmoItem.Clear;
mmoItem.Lines.LoadFromFile('C:\HMI G3\Item.txt'); //

  scpDemo.InsertList.AddStrings(mmoInsert.Lines);
  scpDemo.ItemList.AddStrings(mmoItem.Lines);
  LookupList := TStringList.Create;
  LookupList.Add('L1');
  LookupList.Add('L2');
  LookupList.Add('L3');
  LookupList.Add('L4');
  LookupList.Add('L5');
  LookupList.Add('L6');
  LookupList.Add('L7');
  LookupList.Add('L8');
  LookupList.Add('L9');
  LookupList.Add('L10');
  LookupList.Add('L11');
  LookupList.Add('L12');
  LookupList.Add('L13');
  LookupList.Add('L14');
  LookupList.Add('L15');
  LookupList.Add('L16');
  LookupList.Add('L17');
  LookupList.Add('L18');
  LookupList.Add('L19');
  LookupList.Add('L20');
  LookupList.Add('GC1');
  LookupList.Add('GC2');
  LookupList.Add('GC3');
  LookupList.Add('GC4');
  LookupList.Add('GC5');
  LookupList.Add('GC6');
  LookupList.Add('GC7');
  LookupList.Add('GC8');
  LookupList.Add('GC9');
  LookupList.Add('GC10');
  LookupList.Add('CX1_I');
  LookupList.Add('CX2_I');
  LookupList.Add('CX3_I');
  LookupList.Add('CX4_I');
  LookupList.Add('CX5_I');
  LookupList.Add('CX6_I');
  LookupList.Add('CX7_I');
  LookupList.Add('CX8_I');
  LookupList.Add('CX9_I');
  LookupList.Add('CX10_I');
  LookupList.Add('CX11_I');
  LookupList.Add('CX12_I');
  LookupList.Add('CX13_I');
  LookupList.Add('CX14_I');
  LookupList.Add('CX15_I');
  LookupList.Add('EX1_I');
  LookupList.Add('EX2_I');
  LookupList.Add('EX3_I');
  LookupList.Add('EX4_I');
  LookupList.Add('EX5_I');
  LookupList.Add('EX6_I');
  LookupList.Add('EX7_I');
  LookupList.Add('EX8_I');
  LookupList.Add('EX9_I');
  LookupList.Add('EX10_I');
  LookupList.Add('EX11_I');
  LookupList.Add('EX12_I');
  LookupList.Add('EX13_I');
  LookupList.Add('EX14_I');
  LookupList.Add('EX15_I');
  LookupList.Add('TON1_I');
  LookupList.Add('TON2_I');
  LookupList.Add('TON3_I');
  LookupList.Add('TON4_I');
  LookupList.Add('TON5_I');
  LookupList.Add('TON6_I');
  LookupList.Add('TON7_I');
  LookupList.Add('TON8_I');
  LookupList.Add('TON9_I');
  LookupList.Add('TON10_I');
  LookupList.Add('TON11_I');
  LookupList.Add('TON12_I');
  LookupList.Add('TON13_I');
  LookupList.Add('TON14_I');
  LookupList.Add('TON15_I');
  LookupList.Add('TON16_I');
  LookupList.Add('TON17_I');
  LookupList.Add('TON18_I');
  LookupList.Add('TON19_I');
  LookupList.Add('TON20_I');
  LookupList.Add('TD11_I');
  LookupList.Add('TD12_I');
  LookupList.Add('TD13_I');
  LookupList.Add('TD14_I');
  LookupList.Add('TD15_I');
  LookupList.Add('TD16_I');
  LookupList.Add('TD17_I');
  LookupList.Add('TD18_I');
  LookupList.Add('TD19_I');
  LookupList.Add('TD20_I');
  LookupList.Add('TD21_I');
  LookupList.Add('TD22_I');
  LookupList.Add('TD23_I');
  LookupList.Add('TD24_I');
  LookupList.Add('TD25_I');
  LookupList.Add('TD26_I');
  LookupList.Add('TD27_I');
  LookupList.Add('AI1_I');
  LookupList.Add('AI2_I');
  LookupList.Add('AI3_I');
  LookupList.Add('AI4_I');
  LookupList.Add('AI5_I');
  LookupList.Add('AI6_I');
  LookupList.Add('AI7_I');
  LookupList.Add('AI8_I');
  LookupList.Add('AI9_I');
  LookupList.Add('AI10_I');
  LookupList.Add('SOM_I');
  LookupList.Add('CAM_I');
  LookupList.Add('GIF1_I');
  LookupList.Add('GIF2_I');
  LookupList.Add('GIF3_I');
  LookupList.Add('GIF4_I');
  LookupList.Add('GIF5_I');
  LookupList.Add('GIF6_I');
  LookupList.Add('GIF7_I');
  LookupList.Add('TX_I');
  LookupList.Add('RELATORIO');
  LookupList.Add('MENSSAGEM');
  LookupList.Add('ALARME');
  LookupList.Add('SEGURANCA');
  LookupList.Add('LIMPA_RELATORIOS');
  LookupList.Add('LIMPA_SENHA');
  LookupList.Add('JANELA');
  LookupList.Add('SALVA_RELATORIO');
  LookupList.Add('SALVA_MENSSAGEM');
  LookupList.Add('SALVA_ALARME');
  LookupList.Add('ENTRADA');
  LookupList.Add('TAG_I');


    scpParams.EndOfTokenChr := '';
end;

procedure TSimpleIDEMainForm.FormDestroy(Sender: TObject);
begin
  fDebugger.Free;
end;

procedure TSimpleIDEMainForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if fDebugger.IsRunning then begin
    fDebugger.Stop;
    CanClose := FALSE;
  end;
end;

procedure TSimpleIDEMainForm.SynEditorSpecialLineColors(Sender: TObject;
  Line: Integer; var Special: Boolean; var FG, BG: TColor);
var
  LI: TDebuggerLineInfos;
begin
 // if fDebugger <> nil then begin
    if True then begin

    LI := fDebugger.GetLineInfos(Line);
    if dlCurrentLine in LI then begin
      Special := TRUE;
      FG := clWhite;
      BG := clBlue;
    end else if dlBreakpointLine in LI then begin
      Special := TRUE;
      FG := clWhite;
      if dlExecutableLine in LI then
        BG := clRed
      else
        BG := clGray;
    end;
  end;

end;

procedure TSimpleIDEMainForm.ToolButton2Click(Sender: TObject);
begin
SynEditor.Lines.LoadFromFile('C:\HMI G3\original.pas');

end;

procedure TSimpleIDEMainForm.ToolButton3Click(Sender: TObject);
begin
SynEditor.Lines.SaveToFile(edit2.Text);
end;

procedure TSimpleIDEMainForm.ToolButton4Click(Sender: TObject);
begin
SynEditor.Lines.LoadFromFile(edit2.Text);
end;

procedure TSimpleIDEMainForm.DebuggerBreakpointChange(Sender: TObject;
  ALine: integer);
begin
  if (ALine >= 1) and (ALine <= SynEditor.Lines.Count) then
  begin
    SynEditor.InvalidateGutterLine(ALine);
    SynEditor.InvalidateLine(ALine);
  end
  else
    SynEditor.Invalidate;
end;

procedure TSimpleIDEMainForm.DebuggerCurrentLineChange(Sender: TObject);
begin
  if (fDebugger <> nil) and not fDebugger.IsRunning then
    SetCurrentLine(fDebugger.CurrentLine)
  else
    SetCurrentLine(-1);
end;

procedure TSimpleIDEMainForm.DebuggerStateChange(Sender: TObject; OldState,
  NewState: TDebuggerState);
var
  s: string;
begin
  case NewState of
    dsRunning: s := 'Program is running';
    dsPaused: s := 'Program is paused';
  else
    s := 'Ready';
  end;
  Statusbar.SimpleText := ' ' + s;
end;

procedure TSimpleIDEMainForm.DebuggerYield(Sender: TObject);
begin
  UpdateActions;
  Application.ProcessMessages;
end;

procedure TSimpleIDEMainForm.PaintGutterGlyphs(ACanvas: TCanvas; AClip: TRect;
  FirstLine, LastLine: integer);
var
  LH, X, Y: integer;
  LI: TDebuggerLineInfos;
  ImgIndex: integer;
begin
  if fDebugger <> nil then
  begin
    FirstLine := SynEditor.RowToLine(FirstLine);
    LastLine := SynEditor.RowToLine(LastLine);
    X := 14;
    LH := SynEditor.LineHeight;
    while FirstLine <= LastLine do
    begin
      Y := (LH - imglGutterGlyphs.Height) div 2
           + LH * (SynEditor.LineToRow(FirstLine) - SynEditor.TopLine);
      LI := fDebugger.GetLineInfos(FirstLine);
      if dlCurrentLine in LI then begin
        if dlBreakpointLine in LI then
          ImgIndex := 2
        else
          ImgIndex := 1;
      end else if dlExecutableLine in LI then begin
        if dlBreakpointLine in LI then
          ImgIndex := 3
        else
          ImgIndex := 0;
      end else begin
        if dlBreakpointLine in LI then
          ImgIndex := 4
        else
          ImgIndex := -1;
      end;
      if ImgIndex >= 0 then
        imglGutterGlyphs.Draw(ACanvas, X, Y, ImgIndex);
      Inc(FirstLine);
    end;
  end;
end;

procedure TSimpleIDEMainForm.scpParamsExecute(Kind: TSynCompletionType;
  Sender: TObject; var CurrentInput: string; var x, y: Integer;
  var CanExecute: Boolean);
{
var locline, lookup: String;
    TmpX, savepos, StartX,
    ParenCounter,
    TmpLocation    : Integer;
    FoundMatch     : Boolean;  }
    begin
  //Param Completion is different than Code Completion.  We can't just use
  //the string passed to us we have to figure out what they are looking for,
  //which is language dependant For this demo, I assume that it has to be on the
  //*same* line, then do some paren checking.  For the sake of the demo, the
  //function will be the word directly before the paren.  In other languages you
  //would want to do something like grab everything before the last end of
  //statement char (like in ObjectPascal it's the ';' char).  It *does* support
  //embedded functions (Hense the paren checking).  In this case, commas are the
  //delimiter so they are incremented accordingly.

  //Also everything is hard coded in.  You will want to have some kind of
  //structure that you are using instead of hard coding the parameters in

  with TSynCompletionProposal(Sender).Editor do
  begin
    locLine := LineText;

    //go back from the cursor and find the first open paren
    TmpX := CaretX;
    if TmpX > length(locLine) then
      TmpX := length(locLine)
    else dec(TmpX);
    FoundMatch := False;
    TmpLocation := 0;
    while (TmpX > 0) and not(FoundMatch) do
    begin
      if LocLine[TmpX] = ',' then
      begin
        inc(TmpLocation);
        dec(TmpX);
      end else if LocLine[TmpX] = ')' then
      begin
        //We found a close, go till it's opening paren
        ParenCounter := 1;
        dec(TmpX);
        while (TmpX > 0) and (ParenCounter > 0) do
        begin
          if LocLine[TmpX] = ')' then inc(ParenCounter)
          else if LocLine[TmpX] = '(' then dec(ParenCounter);
          dec(TmpX);
        end;
        if TmpX > 0 then dec(TmpX);  //eat the open paren
      end else if locLine[TmpX] = '(' then
      begin
        //we have a valid open paren, lets see what the word before it is
        StartX := TmpX;
        while (TmpX > 0) and not(locLine[TmpX] in TSynValidStringChars) do
          Dec(TmpX);
        if TmpX > 0 then
        begin
          SavePos := TmpX;
          While (TmpX > 0) and (locLine[TmpX] in TSynValidStringChars) do
            dec(TmpX);
          inc(TmpX);
          lookup := Uppercase(Copy(LocLine, TmpX, SavePos - TmpX + 1));
          FoundMatch := LookupList.IndexOf(Lookup) > -1;
          if not(FoundMatch) then
          begin
            TmpX := StartX;
            dec(TmpX);
          end;
        end;
      end else dec(TmpX)
    end;
  end;

  CanExecute := FoundMatch;

  if CanExecute then
  begin
    TSynCompletionProposal(Sender).Form.CurrentIndex := TmpLocation;
    if Lookup <> TSynCompletionProposal(Sender).PreviousToken then
    begin
      TSynCompletionProposal(Sender).ItemList.Clear;

      if Lookup = 'L1' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L2' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L3' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L4' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L5' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L6' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L7' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L8' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L9' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L10' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L11' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L12' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L13' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L14' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L15' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L16' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L17' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L18' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L19' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'L20' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'GC1' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'GC2' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'GC3' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'GC4' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'GC5' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'GC6' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'GC7' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'GC8' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'GC9' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'GC10' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'CX1_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'CX2_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'CX3_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'CX4_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'CX5_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'CX6_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'CX7_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'CX8_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'CX9_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'CX10_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'CX11_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'CX12_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'CX13_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'CX14_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'CX15_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'EX1_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'EX2_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'EX3_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'EX4_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'EX5_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'EX6_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'EX7_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'EX8_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'EX9_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'EX10_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'EX11_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'EX12_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'EX13_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'EX14_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'EX15_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: string"');
      end else if Lookup = 'TON1_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON2_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON3_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON4_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON5_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON6_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON7_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON8_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON9_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON10_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON11_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON12_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON13_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON14_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON15_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON16_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON17_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON18_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON19_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TON20_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD11_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD12_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD13_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD14_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD15_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD16_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD17_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD18_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD19_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD20_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD21_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD22_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD23_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD24_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD25_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD26_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'TD27_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'AI1_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'AI2_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'AI3_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'AI4_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'AI5_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'AI6_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'AI7_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'AI8_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'AI9_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'AI10_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: integer"');
      end else if Lookup = 'SOM_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"Som_n: integer"');
      end else if Lookup = 'CAM_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"CAM_F: string","CAM_N: integer"');
      end else if Lookup = 'GIF1_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: boolean"');
      end else if Lookup = 'GIF2_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: boolean"');
      end else if Lookup = 'GIF3_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: boolean"');
      end else if Lookup = 'GIF4_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: boolean"');
      end else if Lookup = 'GIF5_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: boolean"');
      end else if Lookup = 'GIF6_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: boolean"');
      end else if Lookup = 'GIF7_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"A: boolean"');
      end else if Lookup = 'TX_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"TX_A: string","TX_B: integer"');
      end else if Lookup = 'RELATORIO' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"relato: string","nl: integer"');
      end else if Lookup = 'MENSSAGEM' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"menssage: string","ml: integer"');
      end else if Lookup = 'ALARME' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"alarme: string","al: integer"');
      end else if Lookup = 'SEGURANCA' then
      begin
       TSynCompletionProposal(Sender).ItemList.Add('"receita: boolean","enviar: boolean","relatorio: boolean","v_relatorio: boolean"');
      end else if Lookup = 'LIMPA_RELATORIOS' then
      begin
       TSynCompletionProposal(Sender).ItemList.Add('"usuario_c: boolean","menssagem_c: boolean","alarme_c: boolean"');
      end else if Lookup = 'LIMPA_SENHA' then
      begin
       TSynCompletionProposal(Sender).ItemList.Add('"senha: integer","comando: integer"');
      end else if Lookup = 'JANELA' then
      begin
       TSynCompletionProposal(Sender).ItemList.Add('"Form: string","Lab1: string","Lab1_v: Boolean","Lab2: string","Lab2_v: boolean","Lab3: string","Lab3_v: boolean","Bot1: string","Bot1_v: boolean","Bot2: string","Bot2_v: boolean","Bot3: string","Bot3_v: boolean","Show: Integer"');
      end else if Lookup = 'SALVA_RELATORIO' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"Local_relatorio: string","Comando_salvar: integer"');
      end else if Lookup = 'SALVA_MENSSAGEM' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"Local_relatorio_M: string","Comando_salvar_m: integer"');
      end else if Lookup = 'SALVA_ALARME' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"Local_relatorio_A: string","Comando_salvar_a: integer"');
      end else if Lookup = 'ENTRADA' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"Tile: string","S_tile: string" ): string;');
      end else if Lookup = 'TAG_I' then
      begin
        TSynCompletionProposal(Sender).ItemList.Add('"TAG_Numero: Integer","TAG_Valor: string" ): string;');
      end;










    end;
  end else TSynCompletionProposal(Sender).ItemList.Clear;
end;
// TSynCompletionProposal(Sender).ItemList.Add('"FirstParam: integer", "SecondParam: integer", "ThirdParam: string"');
procedure TSimpleIDEMainForm.SetCurrentLine(ALine: integer);
begin
  if fCurrentLine <> ALine then
  begin
    SynEditor.InvalidateGutterLine(fCurrentLine);
    SynEditor.InvalidateLine(fCurrentLine);
    fCurrentLine := ALine;
    if (fCurrentLine > 0) and (SynEditor.CaretY <> fCurrentLine) then
      SynEditor.CaretXY := BufferCoord(1, fCurrentLine);
    SynEditor.InvalidateGutterLine(fCurrentLine);
    SynEditor.InvalidateLine(fCurrentLine);
  end;
end;

procedure TSimpleIDEMainForm.actDebugRunExecute(Sender: TObject);
begin
 // fDebugger.Run;
  Edit1.Text:='1';
end;

procedure TSimpleIDEMainForm.actDebugRunUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled := (fDebugger <> nil) and fDebugger.CanRun;
end;

procedure TSimpleIDEMainForm.actDebugStepExecute(Sender: TObject);
begin
  fDebugger.Step;
end;

procedure TSimpleIDEMainForm.actDebugStepUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled := (fDebugger <> nil) and fDebugger.CanStep;
end;

procedure TSimpleIDEMainForm.actDebugGotoCursorExecute(Sender: TObject);
begin
  fDebugger.GotoCursor(SynEditor.CaretY);
end;

procedure TSimpleIDEMainForm.actDebugGotoCursorUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled := (fDebugger <> nil)
    and fDebugger.CanGotoCursor(SynEditor.CaretY);
end;

procedure TSimpleIDEMainForm.actDebugPauseExecute(Sender: TObject);
begin
  fDebugger.Pause;
  Edit1.Text:='1';
end;

procedure TSimpleIDEMainForm.actDebugPauseUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled := (fDebugger <> nil) and fDebugger.CanPause;
end;

procedure TSimpleIDEMainForm.actDebugStopExecute(Sender: TObject);
begin
  fDebugger.Stop;
end;

procedure TSimpleIDEMainForm.actDebugStopUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled := (fDebugger <> nil) and fDebugger.CanStop;
end;

procedure TSimpleIDEMainForm.actToggleBreakpointExecute(Sender: TObject);
begin
  fDebugger.ToggleBreakpoint(SynEditor.CaretY);
end;

procedure TSimpleIDEMainForm.actToggleBreakpointUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled := fDebugger <> nil;
end;

procedure TSimpleIDEMainForm.actClearAllBreakpointsExecute(
  Sender: TObject);
begin
  fDebugger.ClearAllBreakpoints;
end;

procedure TSimpleIDEMainForm.actClearAllBreakpointsUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled := (fDebugger <> nil)
    and fDebugger.HasBreakpoints;
end;

procedure TSimpleIDEMainForm.SynEditorGutterClick(Sender: TObject;
  Button: TMouseButton; X, Y, Line: Integer; Mark: TSynEditMark);
begin

  if fDebugger <> nil then
    fDebugger.ToggleBreakpoint(SynEditor.RowToLine(Line));  
end;

end.

  
